<?php

namespace Drupal\Tests\private_message_queue\Kernel;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\KernelTests\KernelTestBase;
use Drupal\private_message\Entity\PrivateMessage;
use Drupal\private_message\Entity\PrivateMessageThread;
use Drupal\private_message_queue\Plugin\QueueWorker\PrivateMessageQueue;
use Drupal\Tests\user\Traits\UserCreationTrait;

/**
 * @group private_message_queue
 */
class PrivateMessageQueueTest extends KernelTestBase {

  use UserCreationTrait;

  protected static $modules = [
    'filter',
    'private_message',
    'private_message_queue',
    'system',
    'text',
    'user',
  ];

  /**
   * @var \Drupal\Core\Queue\QueueInterface
   */
  private $queue;

  /**
   * @var \Drupal\private_message_queue\Service\PrivateMessageQueuer
   */
  private $queuerService;

  private $privateMessageService;

  protected function setUp() {
    parent::setUp();

    $this->installEntitySchema('pm_thread_access_time');
    $this->installEntitySchema('pm_thread_delete_time');
    $this->installEntitySchema('private_message');
    $this->installEntitySchema('private_message_thread');
    $this->installEntitySchema('user');

    $this->installSchema('system', ['sequences']);

    $this->queue = \Drupal::queue('private_message_queue');
    $this->queue->createQueue();

    $this->queuerService = \Drupal::service('private_message_queue.queuer');

    $this->privateMessageService = \Drupal::service('private_message.service');
  }

  public function testProcessItem() {
    $this->assertFalse($this->queue->numberOfItems());

    $owner = $this->createUser();

    $recipient1 = $this->createUser();
    $recipient2 = $this->createUser();
    $recipient3 = $this->createUser();

    $this->queuerService->queue([$recipient1], 'Message 1', $owner);

    $this->queuerService->queue([$recipient2, $recipient3], 'Message 2', $owner);

    $this->assertEquals(2, $this->queue->numberOfItems());

    $item1 = $this->queue->claimItem();
    $this->assertEquals('Message 1', $item1->data['message']);
    $this->assertEquals(1, $item1->data['owner']->id());
    $this->assertEquals([$recipient1], $item1->data['recipients']);

    $item2 = $this->queue->claimItem();
    $this->assertEquals('Message 2', $item2->data['message']);
    $this->assertEquals(1, $item2->data['owner']->id());
    $this->assertEquals([$recipient2, $recipient3], $item2->data['recipients']);

    $plugin_id = 'private_message_queue';
    $plugin_definition = [
      'id' => $plugin_id,
      'class' => PrivateMessageQueue::class,
      'provider' => 'private_message',
    ];

    $worker = new PrivateMessageQueue([], $plugin_id, $plugin_definition, $this->privateMessageService);
    $worker->processItem($item1->data);
    $worker->processItem($item2->data);

    /** @var \Drupal\private_message\Entity\PrivateMessageThread[] $threads */
    $threads = PrivateMessageThread::loadMultiple([1, 2]);

    $owner = $this->reloadEntity($owner);
    $recipient1 = $this->reloadEntity($recipient1);
    $recipient2 = $this->reloadEntity($recipient2);
    $recipient3 = $this->reloadEntity($recipient3);

    $this->assertCount(2, $threads[1]->getMembers());
    $this->assertEquals([$recipient1, $owner], $threads[1]->getMembers());
    /** @var \Drupal\private_message\Entity\PrivateMessage[] $messages */
    $messages = $threads[1]->getMessages();
    $this->assertNotNull($messages[0]);
    $this->assertEquals('Message 1', $messages[0]->getMessage());

    $this->assertCount(3, $threads[2]->getMembers());
    $this->assertEquals([$recipient2, $recipient3, $owner], $threads[2]->getMembers());
    /** @var \Drupal\private_message\Entity\PrivateMessage[] $messages */
    $messages = $threads[2]->getMessages();
    $this->assertNotNull($messages[0]);
    $this->assertEquals('Message 2', $messages[0]->getMessage());
  }

  /**
   * Reload an entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   An entity to reload.
   *
   * @return mixed
   */
  private function reloadEntity(EntityInterface $entity) {
    $this->entityManager = \Drupal::service('entity.manager');

    $controller = $this->entityManager->getStorage($entity->getEntityTypeId());
    $controller->resetCache([$entity->id()]);

    return $controller->load($entity->id());
  }

}
